/*
--------------------------AUTHORS--------------------------

Aitthikorn Poonsiri     6110500640
Naboon Srisawasdi       6110503339

Computer Engineering, Kasetsart University

***This project is the part of 'Practicum in CPE' Course***

*/


#define F_CPU 16000000
#define FASTLED_ALLOW_INTERRUPTS 0
#define FASTLED_INTERRUPT_RETRY_COUNT 1

#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include <FastLED.h>


#include "peri.h"
#include "usbdrv.h"

#define RQ_SET_LED         0
#define RQ_SET_LED_VALUE   1
#define RQ_GET_SWITCH      2
#define RQ_STATIC_WHITE    3
#define RQ_STATIC_RAINBOW  4
#define RQ_VU              5
#define RQ_TWOSIDE_VU      6
#define RQ_SPECTRUM        7
#define RQ_PEAK_BULLET     8
#define RQ_PULSE           9

#define LED_PIN PD0
#define LED_TYPE WS2812
#define NUM_LEDS 24
#define COLOR_ORDER GRB

struct CRGB VUPallete[NUM_LEDS];
struct CRGB leds[NUM_LEDS];
struct CRGB rainbow[NUM_LEDS];


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint8_t switch_state;

    if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t led_state = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];
        set_led(led_no, led_state);
        return 0;
    }
    else if (rq->bRequest == RQ_SET_LED_VALUE)
    {
        uint8_t led_value = rq->wValue.bytes[0];
        set_led_value(led_value);
        return 0;
    }
    else if (rq->bRequest == RQ_GET_SWITCH)
    {
        switch_state = SWITCH_PRESSED();

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }
    else if(rq->bRequest == RQ_STATIC_WHITE)
    {
      fill_solid(leds, NUM_LEDS, CRGB::White);
      FastLED.setBrightness(100);
      FastLED.show();
      return 0;
    }
    else if(rq->bRequest == RQ_STATIC_RAINBOW)
    {
      fill_rainbow(leds, NUM_LEDS, HUE_PINK, 11);
      FastLED.setBrightness(100);
      FastLED.show();
      return 0;
    }
    else if(rq->bRequest == RQ_VU)
    {
      uint8_t N = rq->wValue.bytes[0];
      uint8_t i = 0;
      for(i=0; i<NUM_LEDS; i++){
        if(i < N) leds[i] = VUPallete[i];
        else leds[i].setRGB(0, 0, 0);
      }
      FastLED.setBrightness(100);
      FastLED.show();
      return 0;
    }
    else if(rq->bRequest == RQ_TWOSIDE_VU)
    {
      uint8_t N = rq->wValue.bytes[0];
      uint8_t i = 0;
      for(i=0; i<NUM_LEDS/2; i++){
        int l, r;
        l = NUM_LEDS/2-i-1;
        r = NUM_LEDS/2+i;
        if(i < N){
          leds[l] = VUPallete[l];
          leds[r] = VUPallete[r];
        }
        else{
          leds[l].setRGB(0, 0, 0);
          leds[r].setRGB(0, 0, 0);
        }
      }
      FastLED.setBrightness(100);
      FastLED.show();
      return 0;
    }
    else if(rq->bRequest == RQ_SPECTRUM)
    {
      uint8_t spectrum = rq->wValue.bytes[0], ledState;
      int i = 0, ledIndex;
      for(i=0; i<8; i++){
        ledIndex = 1 + (3 * i);
        ledState = (spectrum >> i) & 1;
        if(ledState){
          leds[ledIndex] = rainbow[ledIndex];
          leds[ledIndex-1] = rainbow[ledIndex-1];
          leds[ledIndex+1] = rainbow[ledIndex+1];
          leds[ledIndex-1].fadeToBlackBy(192);
          leds[ledIndex+1].fadeToBlackBy(192);
        }
        else{
          leds[ledIndex].fadeToBlackBy(128);
          leds[ledIndex-1].fadeToBlackBy(128);
          leds[ledIndex+1].fadeToBlackBy(128);
        }
      }
      FastLED.show();
      return 0;
    }
    else if(rq->bRequest == RQ_PEAK_BULLET){
      uint8_t bullet = rq->wValue.bytes[0];
      int i=0;
      for(i=0; i<NUM_LEDS/2-1; i++){
        int l, r;
        l = i;
        r = NUM_LEDS-i-1;
        
        if(leds[l+1].r != 0 or leds[l+1].g != 0 or leds[l+1].b != 0){
          leds[l] = VUPallete[l];
          leds[l+1] = CRGB::Black;
          leds[r] = VUPallete[r];
          leds[r-1] = CRGB::Black;
        }
        else{
          leds[l] = CRGB::Black;
          leds[r] = CRGB::Black;
        }
      }
      if(bullet){
        leds[NUM_LEDS/2-1] = VUPallete[NUM_LEDS/2-1];
        leds[NUM_LEDS/2] = VUPallete[NUM_LEDS/2];
      }
      FastLED.show();
      return 0;
    }
    else if(rq->bRequest == RQ_PULSE){
      uint8_t pulse = rq->wValue.bytes[0];
      if(pulse) fill_rainbow(leds, NUM_LEDS, HUE_PINK, 11);
      else fadeToBlackBy(leds, NUM_LEDS, 192);
      FastLED.show();
      return 0;
      
    }
    /* default for not implemented requests: return no data back to host */
    return 0;
}




void setup(){
    
    init_peri();

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();
    
    /* add LED strip */
    LEDS.addLeds<LED_TYPE, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS);

    /* fill Pallete for LED */
    struct CRGB c1, c2, c3;
    c1 = 0x00dbde;
    c2 = CRGB::Crimson;
    c3 = 0xfcb045;
    fill_gradient_RGB(VUPallete, NUM_LEDS, c1, c2, c2, c3);
    fill_rainbow(rainbow, NUM_LEDS, HUE_PINK, 11);
    
    /* init LED strip */
    int hue = 0;
    while(hue <= 255){
      fill_solid(leds, NUM_LEDS, CHSV(hue++, 255, 255));
      FastLED.delay(7);
    }
    FastLED.delay(500);
    FastLED.clear();
    FastLED.delay(20);
    
}


void loop(){
  
    usbPoll();
    
}
