'''
--------------------------AUTHORS--------------------------

Aitthikorn Poonsiri     6110500640
Naboon Srisawasdi       6110503339

Computer Engineering, Kasetsart University

***This project is the part of 'Practicum in CPE' Course***

'''



import usb
from time import sleep


RQ_SET_LED = 0
RQ_SET_LED_VALUE = 1
RQ_GET_SWITCH = 2
RQ_STATIC_WHITE = 3
RQ_STATIC_RAINBOW = 4
RQ_VU = 5
RQ_TWOSIDE_VU = 6
RQ_SPECTRUM = 7
RQ_PEAK_BULLET = 8
RQ_PULSE = 9


####################################
def find_mcu_boards():
    '''
    Find all Practicum MCU boards attached to the machine, then return a list
    of USB device handles for all the boards

    >>> devices = find_mcu_boards()
    >>> first_board = McuBoard(devices[0])
    '''
    boards = [dev for bus in usb.busses()
                  for dev in bus.devices
                  if (dev.idVendor,dev.idProduct) == (0x16c0,0x05dc)]
    return boards

####################################
class McuBoard:
    '''
    Generic class for accessing Practicum MCU board via USB connection.
    '''

    ################################
    def __init__(self, dev):
        self.device = dev
        self.handle = dev.open()

    ################################
    def usb_write(self, request, data=[], index=0, value=0):
        '''
        Send data output to the USB device (i.e., MCU board)
           request: request number to appear as bRequest field on the USB de$
           index: 16-bit value to appear as wIndex field on the USB device
           value: 16-bit value to appear as wValue field on the USB device
        '''
        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_OUT
        self.handle.controlMsg(
                reqType, request, data, value=value, index=index)

    ################################
    def usb_read(self, request, length=1, index=0, value=0):
        '''
        Request data input from the USB device (i.e., MCU board)
           request: request number to appear as bRequest field on the USB de$
           length: number of bytes to read from the USB device
           index: 16-bit value to appear as wIndex field on the USB device
           value: 16-bit value to appear as wValue field on the USB device

        If successful, the method returns a tuple of length specified
        containing data returned from the MCU board.
        '''
        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_IN
        buf = self.handle.controlMsg(
                reqType, request, length, value=value, index=index)
        return buf


####################################
class PeriBoard:

    ################################
    def __init__(self, mcu):
        self.mcu = mcu

    ################################
    def set_led(self, led_no, led_state):
        '''
        Set status of LED led_no on peripheral board to led_state
        (0 => off, 1 => on)
        '''
        self.mcu.usb_write(request=RQ_SET_LED, index=led_no, value=led_state)
        return

    ################################
    def set_led_value(self, value):
        '''
        Display right 3 bits of value on peripheral board's LEDs
        '''
        self.mcu.usb_write(request=RQ_SET_LED_VALUE, value=value)
        return

    ################################
    def get_switch(self):
        '''
        Return a boolean value indicating whether the switch on the peripher$
        board is currently pressed
        '''
        state = self.mcu.usb_read(request=RQ_GET_SWITCH, length=1)
        return state[0] == 1

    ################################
    def get_light(self):
        '''
        Return the current reading of light sensor on peripheral board
        '''
        light = self.mcu.usb_read(request=RQ_GET_LIGHT, length=10)

        return light[0] + (light[1] << 8)

    ################################
    def set_static_white(self):
        '''
            Display static white light on LED strip
        '''
        self.mcu.usb_write(request=RQ_STATIC_WHITE)
        return

    ################################
    def set_static_rainbow(self):
        '''
            Display static rainbow light on LED strip
        '''
        self.mcu.usb_write(request=RQ_STATIC_RAINBOW)
        return

    ################################
    def visualize_vu(self, value):
        '''
            Display LED due to bass peak
        '''
        self.mcu.usb_write(request=RQ_VU, value=value)
        return

    ################################
    def visualize_twoside_vu(self, value):
        '''
            Display VU but start at half
        '''
        self.mcu.usb_write(request=RQ_TWOSIDE_VU, value=value)
        return

    ################################
    def visualize_spectrum(self, value):
        '''
            Display peak of 8 ranges frequency of spectrum
            500 frequency width for each range (60-4000 Hz Frequency)
            3 LEDs for each range
        '''
        self.mcu.usb_write(request=RQ_SPECTRUM, value=value)
        return

    ################################
    def visualize_peak_bullet(self, value):
        '''
            Display running LED start at half
            LED generate by peak of bass frequency
        '''
        self.mcu.usb_write(request=RQ_PEAK_BULLET, value=value)
        return

    ################################
    def visualize_pulse(self, value):
        '''
            Display LED due to peak amplitude of bass frequency
        '''
        self.mcu.usb_write(request=RQ_PULSE, value=value)
        return
