'''
--------------------------AUTHORS--------------------------

Aitthikorn Poonsiri     6110500640
Naboon Srisawasdi       6110503339

Computer Engineering, Kasetsart University

***This project is the part of 'Practicum in CPE' Course***

'''



from practicum import find_mcu_boards, McuBoard, PeriBoard
from time import sleep
from scipy.fftpack import fft
import numpy as np
import pyaudio

CHUNK = 1024 * 2
CHANNELS = 1
RATE = 44100
FORMAT = pyaudio.paInt16

freq = np.linspace(0, RATE, CHUNK)

p = pyaudio.PyAudio()

stream = p.open(
    format = FORMAT,
    channels = CHANNELS,
    rate = RATE,
    input = True,
    frames_per_buffer = CHUNK
)

devs = find_mcu_boards()

if len(devs) == 0:
    print("*** No practicum board found.")
    exit(1)

mcu = McuBoard(devs[0])
print("*** Practicum board found")
print("*** Manufacturer: %s" % \
        mcu.handle.getString(mcu.device.iManufacturer, 256))
print("*** Product: %s" % \
        mcu.handle.getString(mcu.device.iProduct, 256))

peri = PeriBoard(mcu)

def findPeak(d, fromF, toF):
    peak = 0
    for i in range(CHUNK):
        if freq[i] > toF:
            break
        if freq[i] >= fromF:
            peak = max(peak, d[i])
    return peak

def staticWhite():
    print("LED MODE: STATIC WHITE")
    peri.set_static_white()

def staticRainbow():
    print("LED MODE: STATIC RAINBOW")
    peri.set_static_rainbow()

def VUMeter(reactTo):
    peak = findPeak(reactTo, 20, 500)
    print("LED MODE: VU METER            |   PEAK AMPLITUDE:", format(peak, '$
    peak = int(round(peak * 2))
    peak = min(peak, 24)
    peri.visualize_vu(peak)

def twosideVU(reactTo):
    peak = findPeak(reactTo, 60, 500)
    print("LED MODE: TWO-SIDE VU METER   |   PEAK AMPLITUDE:", format(peak, '$
    peak = int(round(peak))
    peak = min(peak, 12)
    peri.visualize_twoside_vu(peak)

def spectrum(reactTo):
    print("LED MODE: REACTIVE SPECTRUM")
    SPECTRUM = 0
    for i in range(1, 9):
        if(i == 1):
            l = 60
        else:
            l = 500 * (i - 1)
        r = 500 * i
        peak = findPeak(reactTo, l, r)
        if(peak > 4+i):
            SPECTRUM = SPECTRUM + (1 << (i - 1))
    peri.visualize_spectrum(SPECTRUM)

def peakBullet(reactTo):
    peak = findPeak(reactTo, 60, 500)
    print("LED MODE: PEAK BULLET         |   PEAK AMPLITUDE:", format(peak, '$
    peri.visualize_peak_bullet(peak > 4.5)

def pulse(reactTo):
    peak = findPeak(reactTo, 60, 500)
    print("LED MODE: PULSE               |   PEAK AMPLITUDE:", format(peak, '$
    peri.visualize_pulse(peak > 4.5)


sw_bounce = False
LED_pattern = 0

while True:
    try:
        sw = peri.get_switch()

        if sw != sw_bounce:
            if sw == True:
                LED_pattern = (LED_pattern + 1) % 7
            sw_bounce = sw

        if LED_pattern >= 2:
            data = stream.read(CHUNK, exception_on_overflow=False)
            data_int = np.frombuffer(data, dtype='h')
            data_fft = fft(data_int)
            data_fft = (np.abs(data_fft[0:CHUNK]) / (32768 * CHUNK)) * 10000

        if LED_pattern == 0:
            staticWhite()
        elif LED_pattern == 1:
            staticRainbow()
        elif LED_pattern == 2:
            VUMeter(data_fft)
        elif LED_pattern == 3:
            twosideVU(data_fft)
        elif LED_pattern == 4:
            spectrum(data_fft)
        elif LED_pattern == 5:
            peakBullet(data_fft)
        elif LED_pattern == 6:
            pulse(data_fft)

    except:
        mcu = McuBoard(devs[0])
        peri = PeriBoard(mcu)

    sleep(0.050)
